@@ -597,14 +597,21 @@ unsigned int nup_xpad = 10;
 unsigned int nup_ypad = 10;
 
 /*
- * --page-label-format
+ * --page-label-format=FORMAT
  *
- * Format in which page labels are printed, default is "short".
+ * Format in which page labels are printed; the default is "short".
  */
 char *page_label_format = "short";
 char page_label_format_buf[256];
 PageLabelFormat page_label;
 
+/*
+ * --ps-level=LEVEL
+ *
+ * The PostScript language level that enscript should use; the default is 2.
+ */
+unsigned int pslevel = 2;
+
 /*
  * --printer-options=OPTIONS
  *
@@ -612,6 +619,15 @@ PageLabelFormat page_label;
  */
 char *printer_options = NULL;
 
+/*
+ * --rotate-even-pages
+ *
+ * Rotate each even-numbered page 180 degrees.  This might be handy in
+ * two-side printing when the resulting pages are bind from some side.
+ * Greetings to Jussi-Pekka Sairanen.
+ */
+int rotate_even_pages = 0;
+
 /*
  * --slice=NUM
  *
@@ -808,7 +824,9 @@ static struct option long_options[] =
   {"nup-xpad",			required_argument,	0, 145},
   {"nup-ypad",			required_argument,	0, 146},
   {"page-label-format",		required_argument,	0, 130},
+  {"ps-level",			required_argument,	0, 149},
   {"printer-options",		required_argument,	0, 139},
+  {"rotate-even-pages",		no_argument,		0, 150},
   {"slice",			required_argument,	0, 140},
   {"toc",			no_argument,		&toc, 1},
   {"word-wrap",			no_argument,		0, 147},
@@ -835,7 +853,7 @@ static struct
     {{"88593", "latin3", NULL},		ENC_ISO_8859_3,		'\n', 8},
     {{"88594", "latin4", NULL},		ENC_ISO_8859_4,		'\n', 8},
     {{"88595", "cyrillic", NULL},	ENC_ISO_8859_5,		'\n', 8},
-    {{"88597", "creek", NULL},		ENC_ISO_8859_7,		'\n', 8},
+    {{"88597", "greek", NULL},		ENC_ISO_8859_7,		'\n', 8},
     {{"ascii", NULL, NULL},		ENC_ASCII, 		'\n', 8},
     {{"asciifise", "asciifi", "asciise"},	ENC_ASCII_FISE,	'\n', 8},
     {{"asciidkno", "asciidk", "asciino"},	ENC_ASCII_DKNO,	'\n', 8},
@@ -1405,6 +1423,7 @@ name             width\theight\tllx\tlly\turx\tury\n\
 		   states_highlight_level,
 		   Fname, Fpt.w, Fpt.h);
 	  input_filter = pp_filter;
+	  input_filter_stdin = "-";
 	}
 
       /* Table of Contents. */
@@ -2177,6 +2196,14 @@ handle_options (int argc, char *argv[])
 	  formfeed_type = FORMFEED_HCOLUMN;
 	  break;
 
+	case 149:		/* PostScript language level */
+	  pslevel = atoi (optarg);
+	  break;
+
+	case 150:		/* rotate even-numbered pages */
+	  rotate_even_pages = 1;
+	  break;
+
 	case '?':		/* Errors found during getopt_long(). */
 	option_error:
 	  fprintf (stderr, _("Try `%s --help' for more information.\n"),
@@ -2302,7 +2329,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   -b, --header=HEADER        set page header\n\
   -B, --no-header            no page headers\n\
   -c, --truncate-lines       cut long lines (default is to wrap)\n\
-  -C, --line-numbers[=START]
+  -C, --line-numbers[=START]\n\
                              precede each line with its line number\n\
   -d                         an alias for option --printer\n\
   -D, --setpagedevice=KEY[:VALUE]\n\
@@ -2385,17 +2412,20 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   --nup-xpad=NUM             set the page x-padding of N-up printing to NUM\n\
   --nup-ypad=NUM             set the page y-padding of N-up printing to NUM\n\
   --page-label-format=FMT    set page label format to FMT\n\
-  --printer-options=OPTIONS  pass extra options to the printer command\n"));
+  --ps-level=LEVEL           set the PostScript language level that enscript\n\
+                             should use\n\
+  --printer-options=OPTIONS  pass extra options to the printer command\n\
+  --rotate-even-pages        rotate even-numbered pages 180 degrees\n"));
 
   printf (_("\
   --slice=NUM                print vertical slice NUM\n\
   --toc                      print table of contents\n\
-  --word-wrap                wrap long lines from word boundaries\n\
   --ul-angle=ANGLE           set underlay text's angle to ANGLE\n\
   --ul-font=NAME             print underlays with font NAME\n\
   --ul-gray=NUM              print underlays with gray value NUM\n\
   --ul-position=POS          set underlay's starting position to POS\n\
   --ul-style=STYLE           print underlays with style STYLE\n\
+  --word-wrap                wrap long lines from word boundaries\n\
 "));
 
   printf (_("\nReport bugs to mtr@iki.fi.\n"));
