@@ -1,13 +1,13 @@
-/* 
+/*
  * Create font map for AFM files.
- * Copyright (c) 1995 Markku Rossi.
+ * Copyright (c) 1995, 1996, 1997 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
 
 /*
  * This file is part of GNU enscript.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -103,7 +103,7 @@ static struct option long_options[] =
  * Global functions.
  */
 
-int 
+int
 main (int argc, char *argv[])
 {
   AFMError error;
@@ -138,9 +138,9 @@ main (int argc, char *argv[])
     {
       int option_index = 0;
       int c;
-      
+
       c = getopt_long (argc, argv, "p:h", long_options, &option_index);
-      if (c == EOF)
+      if (c == -1)
 	break;
 
       switch (c)
@@ -148,7 +148,7 @@ main (int argc, char *argv[])
 	case 'h':		/* help */
 	  usage ();
 	  exit (0);
-	  
+
 	case 'p':		/* output file */
 	  /* Check output file "-". */
 	  if (strcmp (optarg, "-") == 0)
@@ -229,7 +229,7 @@ main (int argc, char *argv[])
 
   if (fname)
     fclose (ofp);
-  
+
   return 0;
 }
 
