@@ -202,6 +202,11 @@ static void divert ();
 /* Paste diverted data to the output and patch the total page counts. */
 static void undivert ();
 
+/*
+ * Handle two-side printing related binding options.  This function is
+ * called once for each even-numbered page.
+ */
+static void handle_two_side_options ();
 
 /*
  * Global variables.
@@ -386,7 +391,8 @@ dump_ps_header ()
     }
 
   /* Page device definitions. */
-  if (count_key_value_set (pagedevice) > 0 || generate_PageSize)
+  if (pslevel >= 2 &&
+      (count_key_value_set (pagedevice) > 0 || generate_PageSize))
     {
       OUTPUT ((cofp, "%% Pagedevice definitions:\n"));
       OUTPUT ((cofp, "gs_languagelevel 1 gt {\n  <<\n    "));
@@ -580,8 +586,14 @@ process_file (char *fname_arg, InputStream *is)
   linewidth = d_output_w / num_columns - 2 * d_output_x_margin
     - line_indent;
 
+  /*
+   * Divert our output to a temp file.  We will re-process it
+   * afterwards to patch, for example, the number of pages in the
+   * document.
+   */
   divert ();
 
+  /* Process this input file. */
   while (!done)
     {
       /* Start a new page. */
@@ -999,6 +1011,7 @@ large for page\n"),
    */
   do_print = 1;
 
+  /* Undivert our output from the temp file to our output stream. */
   undivert ();
 
   /* Table of contents? */
@@ -1930,6 +1943,10 @@ dump_ps_page_header (char *fname, int empty)
 	  /* Page setup. */
 	  OUTPUT ((cofp, "%%%%BeginPageSetup\n_S\n"));
 
+	  if ((total_pages / nup + 1) % 2 == 0)
+	    /* Two-side binding options for the even pages. */
+	    handle_two_side_options ();
+
 #define PRINT_BOUNDING_BOXES 0
 
 #if PRINT_BOUNDING_BOXES
@@ -1997,7 +2014,7 @@ dump_ps_page_header (char *fname, int empty)
 	  ym = nup_subpage / nup_columns;
 	  OUTPUT ((cofp, "%d %d translate\n",
 		   xm * (nup_width + nup_xpad),
-		   ym * (nup_height + nup_ypad) + nup_height));
+		   -(ym * (nup_height + nup_ypad) + nup_height)));
 	}
       OUTPUT ((cofp, "%g dup scale\n", nup_scale));
 
@@ -2008,6 +2025,11 @@ dump_ps_page_header (char *fname, int empty)
   else
     {
       /* No N-up printing. */
+
+      if (total_pages % 2 == 0)
+	/* Two-side binding options for the even pages. */
+	handle_two_side_options ();
+
       if (landscape)
 	OUTPUT ((cofp, "90 rotate\n%d %d translate\n",
 		 media->lly, -media->urx));
@@ -2052,8 +2074,8 @@ dump_ps_page_header (char *fname, int empty)
   if (page_header)
     {
       char *h_left;
-      char *h_center = "";
-      char *h_right = "";
+      char *h_center;
+      char *h_right = NULL;
 
       h_left = format_user_string ("page header", page_header);
       h_center = strchr (h_left, '|');
@@ -2072,8 +2094,10 @@ dump_ps_page_header (char *fname, int empty)
 
       OUTPUT ((cofp, "/user_header_p true def\n"));
       OUTPUT ((cofp, "/user_header_left_str (%s) def\n", h_left));
-      OUTPUT ((cofp, "/user_header_center_str (%s) def\n", h_center));
-      OUTPUT ((cofp, "/user_header_right_str (%s) def\n", h_right));
+      OUTPUT ((cofp, "/user_header_center_str (%s) def\n",
+	       h_center ? h_center : ""));
+      OUTPUT ((cofp, "/user_header_right_str (%s) def\n",
+	       h_right ? h_right : ""));
       xfree (h_left);
     }
   else
@@ -2679,3 +2703,13 @@ undivert ()
 
   cofp = ofp;
 }
+
+
+static void
+handle_two_side_options ()
+{
+  if (rotate_even_pages)
+    /* Rotate page 180 degrees. */
+    OUTPUT ((cofp, "180 rotate\n%d %d translate\n",
+	     -media->w, -media->h));
+}
