@@ -302,6 +302,12 @@ read_config (char *path, char *file)
 	  CHECK_TOKEN ();
 	  page_prefeed = atoi (token2);
 	}
+      else if (MATCH (token, "PostScriptLevel:"))
+	{
+	  token2 = GET_TOKEN (NULL);
+	  CHECK_TOKEN ();
+	  pslevel = atoi (token2);
+	}
       else if (MATCH (token, "Printer:"))
 	{
 	  token2 = GET_TOKEN (NULL);
@@ -1469,7 +1475,9 @@ format_user_string (char *context_name, char *str)
 
 		case 'n':	/* `$n' input file name without directory */
 		  cp = strrchr (fname, '/');
-		  if (cp == NULL)
+		  if (cp)
+		    cp++;
+		  else
 		    cp = fname;
 		  APPEND_STR (cp);
 		  break;
