@@ -24,13 +24,19 @@ typedef union
 #define	tLOCAL	273
 #define	tWHILE	274
 #define	tFOR	275
-#define	tOR	276
-#define	tAND	277
-#define	tEQ	278
-#define	tNE	279
-#define	tGE	280
-#define	tLE	281
-#define	tDIV	282
+#define	tADDASSIGN	276
+#define	tSUBASSIGN	277
+#define	tMULASSIGN	278
+#define	tDIVASSIGN	279
+#define	tOR	280
+#define	tAND	281
+#define	tEQ	282
+#define	tNE	283
+#define	tGE	284
+#define	tLE	285
+#define	tDIV	286
+#define	tPLUSPLUS	287
+#define	tMINUSMINUS	288
 
 
 extern YYSTYPE yylval;
