@@ -1,4 +1,4 @@
-/* 
+/*
  * Primitive procedures for states.
  * Copyright (c) 1997 Markku Rossi.
  *
@@ -7,7 +7,7 @@
 
 /*
  * This file is part of GNU enscript.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -30,34 +30,35 @@
  * Types and definitions.
  */
 
-#define DEFUN(prim) 		\
-static Node * 			\
-prim (prim_name, args, env)  	\
-     char *prim_name; 		\
-     List *args; 		\
-     Environment *env;
+#define DEFUN(prim) 			\
+static Node * 				\
+prim (prim_name, args, env, linenum)  	\
+     char *prim_name; 			\
+     List *args; 			\
+     Environment *env;			\
+     unsigned int linenum;
 
-#define NEED_ARG()							\
-do {									\
-  if (arg == NULL)							\
-    {									\
-      fprintf (stderr, _("%s: primitive `%s': too few arguments\n"),	\
-	       program, prim_name);					\
-      exit (1);								\
-    }									\
+#define NEED_ARG()						\
+do {								\
+  if (arg == NULL)						\
+    {								\
+      fprintf (stderr, _("%s:%d: %s: too few arguments\n"),	\
+	       defs_file, linenum, prim_name);			\
+      exit (1);							\
+    }								\
 } while (0)
 
-#define LAST_ARG()							\
-  do {									\
-    if (arg != NULL)							\
-      {									\
-	fprintf (stderr, _("%s: primitive `%s': too many arguments\n"),	\
-		 program, prim_name);					\
-	exit (1);							\
-      }									\
+#define LAST_ARG()						\
+  do {								\
+    if (arg != NULL)						\
+      {								\
+	fprintf (stderr, _("%s:%d: %s: too many arguments\n"),	\
+		 defs_file, linenum, prim_name);		\
+	exit (1);						\
+      }								\
   } while (0)
 
-#define MATCH_ARG(type) match_arg (prim_name, type, &arg, env)
+#define MATCH_ARG(type) match_arg (prim_name, type, &arg, env, linenum)
 
 #define APPEND(data, len)				\
   do {							\
@@ -78,7 +79,8 @@ do {									\
  * Some forward protypes.
  */
 
-static Node *prim_print ___P ((char *prim_name, List *args, Environment *env));
+static Node *prim_print ___P ((char *prim_name, List *args,
+			       Environment *env, unsigned int linenum));
 
 
 /*
@@ -86,11 +88,12 @@ static Node *prim_print ___P ((char *prim_name, List *args, Environment *env));
  */
 
 static Node *
-match_arg (prim_name, type, argp, env)
+match_arg (prim_name, type, argp, env, linenum)
      char *prim_name;
      NodeType type;
      ListItem **argp;
      Environment *env;
+     unsigned int linenum;
 {
   ListItem *arg = *argp;
   Node *n;
@@ -99,8 +102,8 @@ match_arg (prim_name, type, argp, env)
   n = eval_expr ((Expr *) arg->data, env);
   if (type != nVOID && n->type != type)
     {
-      fprintf (stderr, _("%s: primitive `%s': illegal argument type\n"),
-	       program, prim_name);
+      fprintf (stderr, _("%s:%d: %s: illegal argument type\n"),
+	       defs_file, linenum, prim_name);
       exit (1);
     }
   *argp = arg->next;
@@ -119,8 +122,8 @@ DEFUN (prim_call)
   e = (Expr *) arg->data;
   if (e->type != eSYMBOL)
     {
-      fprintf (stderr, _("%s: primitive `%s': illegal argument type\n"),
-	       program, prim_name);
+      fprintf (stderr, _("%s:%d: %s: illegal argument type\n"),
+	       defs_file, linenum, prim_name);
       exit (1);
     }
   cp = e->u.node->u.sym;
@@ -149,7 +152,7 @@ DEFUN (prim_check_namerules)
       c = (Cons *) i->data;
       n = (Node *) c->car;
 
-      if (re_search (&n->u.re.compiled, current_fname, strlen (current_fname),
+      if (re_search (REGEXP (n), current_fname, strlen (current_fname),
 		     0, strlen (current_fname), NULL) >= 0)
 	{
 	  /* This is it. */
@@ -189,7 +192,7 @@ DEFUN (prim_check_startrules)
       c = (Cons *) i->data;
       n = (Node *) c->car;
 
-      if (re_search (&n->u.re.compiled, inbuf, data_in_buffer, 
+      if (re_search (REGEXP (n), inbuf, data_in_buffer,
 		     0, data_in_buffer, NULL) >= 0)
 	{
 	  /* This is it. */
@@ -225,14 +228,17 @@ DEFUN (prim_concat)
       n = eval_expr ((Expr *) arg->data, env);
       if (n->type != nSTRING)
 	{
-	  fprintf (stderr, _("%s: primitive `%s': illegal argument type\n"),
-		   program, prim_name);
+	  fprintf (stderr, _("%s:%d: %s: illegal argument type\n"),
+		   defs_file, linenum, prim_name);
 	  exit (1);
 	}
 
-      data = (char *) xrealloc (data, len + n->u.str.len);
-      memcpy (data + len, n->u.str.data, n->u.str.len);
-      len += n->u.str.len;
+      if (n->u.str.len > 0)
+	{
+	  data = (char *) xrealloc (data, len + n->u.str.len);
+	  memcpy (data + len, n->u.str.data, n->u.str.len);
+	  len += n->u.str.len;
+	}
       node_free (n);
     }
 
@@ -244,6 +250,54 @@ DEFUN (prim_concat)
 }
 
 
+DEFUN (prim_float)
+{
+  ListItem *arg = args->head;
+  Node *n, *r;
+  char buf[512];
+
+  n = MATCH_ARG (nVOID);
+  LAST_ARG ();
+
+  r = node_alloc (nREAL);
+
+  switch (n->type)
+    {
+    case nVOID:
+    case nREGEXP:
+    case nSYMBOL:
+      r->u.real = 0.0;
+      break;
+
+    case nARRAY:
+      r->u.real = (double) n->u.array.len;
+      break;
+
+    case nSTRING:
+      if (n->u.str.len > sizeof (buf) - 1)
+	r->u.real = 0.0;
+      else
+	{
+	  memcpy (buf, n->u.str.data, n->u.str.len);
+	  buf[n->u.str.len] = '\0';
+	  r->u.real = atof (buf);
+	}
+      break;
+
+    case nINTEGER:
+      r->u.real = (double) n->u.integer;
+      break;
+
+    case nREAL:
+      r->u.real = n->u.real;
+      break;
+    }
+
+  node_free (n);
+  return r;
+}
+
+
 DEFUN (prim_getenv)
 {
   ListItem *arg = args->head;
@@ -253,12 +307,12 @@ DEFUN (prim_getenv)
 
   var = MATCH_ARG (nSTRING);
   LAST_ARG ();
-  
+
   key = (char *) xcalloc (1, var->u.str.len + 1);
   memcpy (key, var->u.str.data, var->u.str.len);
 
   cp = getenv (key);
-  
+
   node_free (var);
   xfree (key);
 
@@ -347,8 +401,8 @@ DEFUN (prim_length)
 	  break;
 
 	default:
-	  fprintf (stderr, _("%s: primitive `%s': illegal argument type\n"),
-		   program, prim_name);
+	  fprintf (stderr, _("%s:%d: %s: illegal argument type\n"),
+		   defs_file, linenum, prim_name);
 	  exit (1);
 	  break;
 	}
@@ -391,7 +445,7 @@ DEFUN (prim_panic)
 {
   fprintf (stderr, _("%s: panic: "), program);
   ofp = stderr;
-  prim_print (prim_name, args, env);
+  prim_print (prim_name, args, env, linenum);
   fprintf (stderr, "\n");
   exit (1);
 
@@ -423,8 +477,8 @@ DEFUN (prim_prereq)
   if (sscanf (cp, "%d.%d.%d", &rver[0], &rver[1], &rver[2]) != 3)
     {
       fprintf (stderr,
-	       _("%s: primitive `%s': malformed version string `%s'\n"),
-	       program, prim_name, cp);
+	       _("%s:%d: %s: malformed version string `%s'\n"),
+	       defs_file, linenum, prim_name, cp);
       exit (1);
     }
 
@@ -512,6 +566,124 @@ DEFUN (prim_print)
 }
 
 
+DEFUN (prim_range)
+{
+  ListItem *arg = args->head;
+  Node *from, *start, *end, *n;
+  int i;
+
+  NEED_ARG ();
+  from = eval_expr ((Expr *) arg->data, env);
+  arg = arg->next;
+
+  start = MATCH_ARG (nINTEGER);
+  end = MATCH_ARG (nINTEGER);
+  LAST_ARG ();
+
+  if (start->u.integer > end->u.integer)
+    {
+      fprintf (stderr,
+	       _("%s:%d: %s: start offset is bigger than end offset\n"),
+	       defs_file, linenum, prim_name);
+      exit (1);
+    }
+
+  if (from->type == nSTRING)
+    {
+      if (end->u.integer > from->u.str.len)
+	{
+	  fprintf (stderr, _("%s:%d: %s: offset out of range\n"),
+		   defs_file, linenum, prim_name);
+	  exit (1);
+	}
+
+      n = node_alloc (nSTRING);
+      n->u.str.len = end->u.integer - start->u.integer;
+      /* +1 to avoid zero allocation */
+      n->u.str.data = (char *) xmalloc (n->u.str.len + 1);
+      memcpy (n->u.str.data, from->u.str.data + start->u.integer,
+	      n->u.str.len);
+    }
+  else if (from->type == nARRAY)
+    {
+      if (end->u.integer > from->u.array.len)
+	{
+	  fprintf (stderr, _("%s:%d: %s: offset out of range\n"),
+		   defs_file, linenum, prim_name);
+	  exit (1);
+	}
+
+      n = node_alloc (nARRAY);
+      n->u.array.len = end->u.integer - start->u.integer;
+      /* +1 to avoid zero allocation */
+      n->u.array.allocated = n->u.array.len + 1;
+      n->u.array.array = (Node **) xcalloc (n->u.array.allocated,
+					    sizeof (Node *));
+
+      for (i = 0; i < n->u.array.len; i++)
+	n->u.array.array[i]
+	  = node_copy (from->u.array.array[i + start->u.integer]);
+    }
+  else
+    {
+      fprintf (stderr, _("%s:%d: %s: illegal argument\n"),
+	       defs_file, linenum, prim_name);
+      exit (1);
+    }
+
+  node_free (from);
+  node_free (start);
+  node_free (end);
+
+  return n;
+}
+
+
+DEFUN (prim_regexp)
+{
+  ListItem *arg = args->head;
+  Node *str, *n;
+
+  str = MATCH_ARG (nSTRING);
+  LAST_ARG ();
+
+  /* Create a new REGEXP node. */
+
+  n = node_alloc (nREGEXP);
+  n->u.re.data = xmalloc (str->u.str.len + 1);
+  n->u.re.len = str->u.str.len;
+  memcpy (n->u.re.data, str->u.str.data, str->u.str.len);
+  n->u.re.data[str->u.str.len] = '\0';
+
+  return n;
+}
+
+
+DEFUN (prim_regexp_syntax)
+{
+  ListItem *arg = args->head;
+  Node *ch, *st;
+  char syntax;
+
+  ch = MATCH_ARG (nINTEGER);
+  st = MATCH_ARG (nINTEGER);
+  LAST_ARG ();
+
+  syntax = (char) st->u.integer;
+  if (syntax != 'w' && syntax != ' ')
+    {
+      fprintf (stderr,
+	       _("%s:%d: %s: illegal regexp character syntax: %c\n"),
+	       defs_file, linenum, prim_name, syntax);
+      exit (1);
+    }
+
+  re_set_character_syntax ((unsigned char) ch->u.integer, syntax);
+
+  return nvoid;
+}
+
+
 DEFUN (prim_regmatch)
 {
   ListItem *arg = args->head;
@@ -525,7 +697,7 @@ DEFUN (prim_regmatch)
   LAST_ARG ();
 
   /* Search for match. */
-  i = re_search (&re->u.re.compiled, str->u.str.data, str->u.str.len,
+  i = re_search (REGEXP (re), str->u.str.data, str->u.str.len,
 		 0, str->u.str.len, &matches);
 
   if (i < 0)
@@ -542,7 +714,7 @@ DEFUN (prim_regmatch)
       current_match_buf = str->u.str.data;
     }
   node_free (re);
-	
+
   n = node_alloc (nINTEGER);
   n->u.integer = (i >= 0);
 
@@ -581,16 +753,16 @@ do_regsubsts (str, re, subst, allp)
   while (1)
     {
       /* Search for match. */
-      i = re_search (&re->u.re.compiled, str->u.str.data, str->u.str.len,
+      i = re_search (REGEXP (re), str->u.str.data, str->u.str.len,
 		     pos, str->u.str.len - pos, &matches);
       if (i < 0)
 	goto out;
-      
+
       num_matches++;
 
       /* Everything before match. */
       APPEND (str->u.str.data + pos, matches.start[0] - pos);
-      
+
       /* Append match. */
       if (!do_expansions_in_substs)
 	APPEND (subst->u.str.data, subst->u.str.len);
@@ -633,7 +805,7 @@ do_regsubsts (str, re, subst, allp)
 	    else
 	      APPEND (subst->u.str.data + i, 1);
 	}
-      
+
       /* Update pos. */
       pos = matches.end[0];
 
@@ -651,7 +823,7 @@ out:
     }
 
   APPEND (str->u.str.data + pos, str->u.str.len - pos);
-  
+
   /* Create result node. */
   node_free (str);
   node_free (re);
@@ -670,7 +842,7 @@ DEFUN (prim_regsub)
 {
   ListItem *arg = args->head;
   Node *str, *re, *subst;
-  
+
   str = MATCH_ARG (nSTRING);
   re = MATCH_ARG (nREGEXP);
   subst = MATCH_ARG (nSTRING);
@@ -684,7 +856,7 @@ DEFUN (prim_regsuball)
 {
   ListItem *arg = args->head;
   Node *str, *re, *subst;
-  
+
   str = MATCH_ARG (nSTRING);
   re = MATCH_ARG (nREGEXP);
   subst = MATCH_ARG (nSTRING);
@@ -693,7 +865,63 @@ DEFUN (prim_regsuball)
   return do_regsubsts (str, re, subst, 1);
 }
 
-
+
+DEFUN (prim_split)
+{
+  ListItem *arg = args->head;
+  Node *re, *str, *n, *n2;
+  int pos, i;
+
+  re = MATCH_ARG (nREGEXP);
+  str = MATCH_ARG (nSTRING);
+  LAST_ARG ();
+
+  /* Create a new array node. */
+  n = node_alloc (nARRAY);
+  n->u.array.allocated = 100;
+  n->u.array.array = (Node **) xcalloc (n->u.array.allocated, sizeof (Node *));
+
+  for (pos = 0; pos < str->u.str.len;)
+    {
+      i = re_search (REGEXP (re), str->u.str.data, str->u.str.len,
+		     pos, str->u.str.len - pos, &re->u.re.matches);
+      if (i < 0)
+	/* No more matches. */
+	break;
+
+      /* Append the string before the first match. */
+      n2 = node_alloc (nSTRING);
+      n2->u.str.len = i - pos;
+      n2->u.str.data = (char *) xmalloc (n2->u.str.len + 1);
+      memcpy (n2->u.str.data, str->u.str.data + pos, n2->u.str.len);
+      pos = re->u.re.matches.end[0];
+
+      /*
+       * Check that at least one item fits after us (no need to check
+       * when appending the last item).
+       */
+      if (n->u.array.len + 1 >= n->u.array.allocated)
+	{
+	  n->u.array.allocated += 100;
+	  n->u.array.array = (Node **) xrealloc (n->u.array.array,
+						 n->u.array.allocated
+						 * sizeof (Node *));
+	}
+      n->u.array.array[n->u.array.len++] = n2;
+    }
+
+  /* Append all the remaining data. */
+  n2 = node_alloc (nSTRING);
+  n2->u.str.len = str->u.str.len - pos;
+  n2->u.str.data = (char *) xmalloc (n2->u.str.len + 1);
+  memcpy (n2->u.str.data, str->u.str.data + pos, n2->u.str.len);
+
+  n->u.array.array[n->u.array.len++] = n2;
+
+  return n;
+}
+
+
 DEFUN (prim_sprintf)
 {
   ListItem *arg = args->head;
@@ -754,8 +982,8 @@ DEFUN (prim_sprintf)
 		{
 		no_match:
 		  fprintf (stderr,
-			   _("%s: primitive `%s': argument %d doesn't match format\n"),
-			   program, prim_name, argument_count);
+			   _("%s:%d: %s: argument %d doesn't match format\n"),
+			   defs_file, linenum, prim_name, argument_count);
 		  exit (1);
 		}
 	      sprintf (ifmt, "%%%s%c", ifmtopts, cp[i]);
@@ -784,8 +1012,8 @@ DEFUN (prim_sprintf)
 	      if (ifmtopts[0] != '\0')
 		{
 		  fprintf (stderr,
-			   _("%s: primitive `%s': no extra options can be specified for %%s\n"),
-			   program, prim_name);
+			   _("%s:%d: %s: no extra options can be specified for %%s\n"),
+			   defs_file, linenum, prim_name);
 		  exit (1);
 		}
 	      APPEND (n->u.str.data, n->u.str.len);
@@ -793,8 +1021,8 @@ DEFUN (prim_sprintf)
 
 	    default:
 	      fprintf (stderr,
-		       _("%s: primitive `%s': illegal type specifier `%c'\n"),
-		       program, prim_name, cp[i]);
+		       _("%s:%d: %s: illegal type specifier `%c'\n"),
+		       defs_file, linenum, prim_name, cp[i]);
 	      exit (1);
 	      break;
 	    }
@@ -808,7 +1036,7 @@ DEFUN (prim_sprintf)
   n = node_alloc (nSTRING);
   n->u.str.len = result_pos;
   n->u.str.data = result;
-  
+
   return n;
 }
 
@@ -854,10 +1082,10 @@ out:
   node_free (s2);
   n = node_alloc (nINTEGER);
   n->u.integer = result;
-  
+
   return n;
 }
- 
+
 
 DEFUN (prim_string)
 {
@@ -878,7 +1106,7 @@ DEFUN (prim_string)
       r->u.str.data = (char *) xcalloc (1, 1);
       r->u.str.len = 0;
       break;
-      
+
     case nSYMBOL:
       r->u.str.len = strlen (n->u.sym);
       r->u.str.data = (char *) xmalloc (r->u.str.len);
@@ -962,7 +1190,7 @@ out:
   node_free (len);
   n = node_alloc (nINTEGER);
   n->u.integer = result;
-  
+
   return n;
 }
 
@@ -980,14 +1208,14 @@ DEFUN (prim_substring)
   if (start->u.integer > end->u.integer)
     {
       fprintf (stderr,
-	       _("%s: primitive `%s': start offset is bigger than end offset\n"),
-	       program, prim_name);
+	       _("%s:%d: %s: start offset is bigger than end offset\n"),
+	       defs_file, linenum, prim_name);
       exit (1);
     }
-  if (start->u.integer > str->u.str.len || end->u.integer > str->u.str.len)
+  if (end->u.integer > str->u.str.len)
     {
-      fprintf (stderr, _("%s: primitive `%s': too large offsets\n"),
-	       program, prim_name);
+      fprintf (stderr, _("%s:%d: %s: offset out of range\n"),
+	       defs_file, linenum, prim_name);
       exit (1);
     }
 
@@ -1021,6 +1249,7 @@ static struct
     {"check_namerules",		prim_check_namerules},
     {"check_startrules",	prim_check_startrules},
     {"concat",			prim_concat},
+    {"float", 			prim_float},
     {"getenv", 			prim_getenv},
     {"int",			prim_int},
     {"length",			prim_length},
@@ -1028,9 +1257,13 @@ static struct
     {"panic", 			prim_panic},
     {"prereq", 			prim_prereq},
     {"print", 			prim_print},
+    {"range", 			prim_range},
+    {"regexp",			prim_regexp},
+    {"regexp_syntax",		prim_regexp_syntax},
     {"regmatch",		prim_regmatch},
     {"regsub",			prim_regsub},
     {"regsuball",		prim_regsuball},
+    {"split",			prim_split},
     {"sprintf",			prim_sprintf},
     {"strcmp",			prim_strcmp},
     {"string",			prim_string},
